﻿using AddinExpress.MSO;
using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace adx_ol_cs_PaneLocation
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("86C0F177-D08F-4F41-96E3-2889A4770292"), ProgId("adxOlPaneLocationCS.AddinModule")]
    public partial class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        private const int WM_User = 1024;
        private const int WM_OpenFormInCurrentFolder = WM_User + 121; // 121 - an arbitrary value > 0

        AddinExpress.OL.ADXOlCachingStrategy currentStrategy;
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
            currentStrategy = AddinExpress.OL.ADXOlCachingStrategy.NewInstanceForEachFolder;
            chkStrategyNewInstanceForEachFolder.Pressed = true;
            adxOlFormsCollectionItem1.Tag = "pre-created";
            adxOlFormsCollectionItem2.Tag = "pre-created";
        }

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance 
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_OnSendMessage(object sender, ADXSendMessageEventArgs e)
        {
            if (e.Message == WM_OpenFormInCurrentFolder)
            {
                Outlook.Explorer explorer = OutlookApp.ActiveExplorer();
                if (explorer != null)
                {
                    // if there's no form in the current explorer, request the ADXolFromsManager to create a new from 
                    AddinExpress.OL.ADXOlForm form = this.adxOlFormsCollectionItem1.GetForm(explorer);
                    if (form == null)
                    {
                        this.adxOlFormsCollectionItem1.ApplyTo(explorer);
                    }
                    Marshal.ReleaseComObject(explorer);
                }
            } 
        }

        private void adxOlFormsManager1_ADXBeforeFormInstanceCreate(object sender, AddinExpress.OL.BeforeFormInstanceCreateEventArgs args)
        {
            Logging.Log.Add("FormsManager.", "Creating a new instance of " + args.Item.FormClassName);
        }

        private void adxRibbonButton1_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            object context = control.Context;
            if (context is Outlook.Explorer)
            {
                string message =
                    "This creates a new ADXOlFormsCollectionItem and adds it to the Items collection of the ADXOlFormsManager component." +
                    "\n" + "\n" +

                    "When the Outlook context changes - e.g. you switch to a folder - the ADXOlFormsManager scans the collection looking for items whose context settings match the context." +
                    " For every such item, the Forms Manager creates a new form instance (of the type that ADXOlFormsCollectionItem specifies) or shows a previously created one." +
                    "\n" + "\n" +

                    "The item's settings - such as the Cached property (see the '" + adxRibbonGroupCachingStrategy.Caption + "' Ribbon group) or CloseButton - control how instances of the specified form type are created." +
                    "\n" + "\n" +

                    "Do you want to have the Close button on form instances created by the new ADXOlFormsCollectionItem?";

                DialogResult dialogResult = MessageBox.Show(message, this.AddinName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (dialogResult == DialogResult.Cancel) return;

                bool showCloseButton = (dialogResult == DialogResult.Yes);

                AddinExpress.OL.ADXOlFormsCollectionItem item = new AddinExpress.OL.ADXOlFormsCollectionItem();
                item.AlwaysShowHeader = adxOlFormsCollectionItem1.AlwaysShowHeader;
                item.Cached = adxOlFormsCollectionItem1.Cached;
                item.CloseButton = showCloseButton;
                item.Enabled = true;
                item.ExplorerAllowedDropRegions = adxOlFormsCollectionItem1.ExplorerAllowedDropRegions;
                item.ExplorerItemTypes = adxOlFormsCollectionItem1.ExplorerItemTypes;
                item.ExplorerLayout = AddinExpress.OL.ADXOlExplorerLayout.LeftSubpane;
                item.FolderName = "*";
                item.FormClassName = adxOlFormsCollectionItem1.FormClassName;
                item.Splitter = adxOlFormsCollectionItem1.Splitter;
                item.SplitterSize = adxOlFormsCollectionItem1.SplitterSize;
                item.UseOfficeThemeForBackground = adxOlFormsCollectionItem1.UseOfficeThemeForBackground;
                item.IsDragDropAllowed = adxOlFormsCollectionItem1.IsDragDropAllowed;
                item.IsHiddenStateAllowed = adxOlFormsCollectionItem1.IsHiddenStateAllowed;
                item.IsMinimizedStateAllowed = adxOlFormsCollectionItem1.IsMinimizedStateAllowed;
                item.IsNormalStateAllowed = adxOlFormsCollectionItem1.IsNormalStateAllowed;
                item.Tag = "created at " + DateTime.Now.ToLongTimeString();
                this.adxOlFormsManager1.Items.Add(item);

                Outlook.Explorer explorer = context as Outlook.Explorer; ;
                item.ApplyTo(explorer);
                Logging.Log.Add("Add-in", "A new ADXOlFormsCollectionItem is created.");
            } 
            else
            {
                string message =
                    "This creates a new ADXOlFormsCollectionItem and adds it to the Items collection of the ADXOlFormsManager component." +
                    "\n" + "\n" +

                    "When the Outlook context changes - e.g. you open a new inspector window - the ADXOlFormsManager scans the collection looking for items whose context settings match the context." +
                    " For every such item, the Forms Manager creates a new form instance (of the type that ADXOlFormsCollectionItem specifies) or shows a previously created one." +
                    "\n" + "\n" +

                    "The item's settings - such as the CloseButton - control how instances of the specified form type are created." +
                    "\n" + "\n" +

                    "Do you want to have the Close button on form instances created by the new ADXOlFormsCollectionItem?";

                DialogResult dialogResult = MessageBox.Show(message, this.AddinName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (dialogResult == DialogResult.Cancel) return;

                bool showCloseButton = (dialogResult == DialogResult.Yes);

                AddinExpress.OL.ADXOlFormsCollectionItem item = new AddinExpress.OL.ADXOlFormsCollectionItem();
                item.AlwaysShowHeader = adxOlFormsCollectionItem2.AlwaysShowHeader;
                item.Cached = adxOlFormsCollectionItem2.Cached;
                item.CloseButton = showCloseButton;
                item.Enabled = true;

                item.InspectorItemTypes = adxOlFormsCollectionItem2.InspectorItemTypes;
                item.InspectorLayout = AddinExpress.OL.ADXOlInspectorLayout.LeftSubpane;
                item.InspectorAllowedDropRegions = adxOlFormsCollectionItem2.InspectorAllowedDropRegions;
                item.FolderName = "*";
                item.FormClassName = adxOlFormsCollectionItem2.FormClassName;
                item.Splitter = adxOlFormsCollectionItem2.Splitter;
                item.SplitterSize = adxOlFormsCollectionItem2.SplitterSize;
                item.UseOfficeThemeForBackground = adxOlFormsCollectionItem2.UseOfficeThemeForBackground;
                item.IsDragDropAllowed = adxOlFormsCollectionItem2.IsDragDropAllowed;
                item.IsHiddenStateAllowed = adxOlFormsCollectionItem2.IsHiddenStateAllowed;
                item.IsMinimizedStateAllowed = adxOlFormsCollectionItem2.IsMinimizedStateAllowed;
                item.IsNormalStateAllowed = adxOlFormsCollectionItem2.IsNormalStateAllowed;
                item.Tag = "created at " + DateTime.Now.ToLongTimeString();
                this.adxOlFormsManager1.Items.Add(item);

                Outlook.Inspector inspector = context as Outlook.Inspector;
                item.ApplyTo(inspector);
                Logging.Log.Add("Add-in", "A new ADXOlFormsCollectionItem is created.");
            }
            Marshal.ReleaseComObject(context);
        }

        private void adxRibbonButton2_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            FormResources form = new FormResources();
            form.ShowDialog();
        }

        private void adxRibbonButton3_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            using (DpiScaling.DPIContextBlock context = new DpiScaling.DPIContextBlock(DpiScaling.DPIHelper.DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2))
            {
                FormLog form = new FormLog();
                form.ShowDialog();
            }

        }

        private void adxRibbonButton4_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            FormAbout form = new FormAbout();
            form.ShowDialog();
        }

        private void chkStrategy_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            if (pressed)
            {
                bool changed = false;
                int tag = (sender as ADXRibbonCheckBox).Tag;
                AddinExpress.OL.ADXOlCachingStrategy newStrategy =
                    (AddinExpress.OL.ADXOlCachingStrategy)Enum.Parse(
                        typeof(AddinExpress.OL.ADXOlCachingStrategy), (tag - 10).ToString()
                    );

                if (currentStrategy != newStrategy)
                {
                    for (int i = 0; i < adxOlFormsManager1.Items.Count; i++)
                    {
                        adxOlFormsManager1.Items[i].Cached = newStrategy;
                        changed = true;
                    }

                    if (!changed) return;
                }

                Outlook.Explorer explorer = OutlookApp.ActiveExplorer();
                if (explorer != null)
                {
                    for (int i = 0; i < adxOlFormsManager1.Items.Count; i++)
                    {
                        AddinExpress.OL.ADXOlForm form = adxOlFormsManager1.Items[i].GetForm(explorer);
                        if (form == null)
                        {
                            // if there's no form in the current explorer, request the ADXolFromsManager to create a new from 
                            adxOlFormsManager1.Items[i].ApplyTo(explorer);
                        }
                        changed = true;
                    }
                    Marshal.ReleaseComObject(explorer);
                }
                currentStrategy = newStrategy;
            }
            this.chkStrategyNone.Invalidate();
            this.chkStrategyNewInstanceForEachFolder.Invalidate();
            this.chkStrategyOneInstanceForAllFolders.Invalidate();
        }

        private void chkStrategy_PropertyChanging(object sender, ADXRibbonPropertyChangingEventArgs e)
        {
            if (e.PropertyType == ADXRibbonControlPropertyType.Pressed)
            {
                if ((sender as ADXRibbonCheckBox).Tag == ((int)currentStrategy + 10))
                {
                    e.Value = true;
                }
                else
                {
                    e.Value = false;
                }                     
            }
        }
    }
}

