using AddinExpress.OL;
using System;
using System.Globalization;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace adx_ol_cs_PaneLocation
{
    public partial class ADXOlFormExplorer : AddinExpress.OL.ADXOlForm
    {
        private static int counter = 0;
        private const string ADXHTMLFileName = "ADXOlFormGeneral.html";
        private adx_ol_cs_PaneLocation.AddinModule theAddinModule;
        private int id = 0;
        
        private bool _refreshCurrentFolderExecuting = false;
        
        public ADXOlFormExplorer()
        {
            InitializeComponent();
            id = counter++;
            this.Text = "ADXOlForm1. ID=" + id.ToString();
            Logging.Log.Add(this.Text, "Constructor");
        }

        #region form events: setting up the controls and closing the form
        private void ADXOlFormExplorer_ADXBeforeFormShow()
        {
            Outlook.Explorer explorer = this.ExplorerObj as Outlook.Explorer;
            if (explorer == null) throw new Exception("ADXOlFormExplorer must be shown in Explorer windows");

            Outlook.MAPIFolder folder = explorer.CurrentFolder;
            string folderPath = "(unknown)";
            try { folderPath = folder.FolderPath; } catch { }
            
            Logging.Log.Add(this.Text, string.Format("ADXBeforeFormShow. folder='{0}'", folderPath));

            theAddinModule = adx_ol_cs_PaneLocation.AddinModule.CurrentInstance;

            this.elementHost1.MinimumSize = new System.Drawing.Size((int)userControl11.MinWidth, (int)userControl11.MinHeight);

            #region populate the list and get the info string
            string[] items = new string[] {
                "LeftSubpane", "TopSubpane", "RightSubpane", "BottomSubpane",
                "LeftReadingPane", "TopReadingPane", "RightReadingPane", "BottomReadingPane",
                "DockLeft", "DockTop", "DockRight", "DockBottom",
                "WebViewPane",
                "BottomOutlookBar", "BottomNavigationPane",
                "BottomTodoBar"};

            userControl11.list.SelectionChanged -= List_SelectionChanged;
            userControl11.PopulateListAndSelectItem(items, this.Item.ExplorerLayout.ToString());
            userControl11.list.SelectionChanged += List_SelectionChanged;

            string formType = "FormClassName = 'ADXOlForm1'";
            string cashingStrategy = "Caching strategy (Cached) = '" + this.Item.Cached.ToString() + "'";
            string currentLayout = "ExplorerLayout = '" + this.Item.ExplorerLayout.ToString() + "'";
            string currentInstance = "Instance Id = " + id.ToString();
            string currentTime = "Created at: " + System.DateTime.Now.ToLongTimeString();
            string itemInfo = "ADXOlFormsCollectionItem: " + this.Item.Tag;
            string dragDrop = "Item.IsDragDropAllowed: " + this.Item.IsDragDropAllowed.ToString();
            userControl11.Info = formType + "\n" + currentInstance + "\n" + currentTime + "\n" + itemInfo + "\n" + dragDrop;
            #endregion
            Logging.Log.Add(this.Text, "Form will be shown in " + currentLayout);
        }

        private void ADXOlFormExplorer_ADXCloseButtonClick(object sender, ADXCloseButtonClickEventArgs e)
        {
            Logging.Log.Add(this.Text, "Close button is clicked");
            string message = "" +
                "You can close the current form instance. \n\n" +
                "Or\n\n" +
                "You can close all from instances created by an ADXOlFromsCollectionItem AND prevent the item from creating other form instances.\n\n" +
                "Click Yes to close this form instance. Close No to disable the corresponing ADXOlFromsCollectionItem.";
            DialogResult dialogResult = MessageBox.Show(message, "What to close?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (dialogResult == DialogResult.Cancel) return;
            if (dialogResult == DialogResult.Yes) e.CloseForm = true;
            if (dialogResult == DialogResult.No) e.DisableItem = true;
        }
        #endregion


        #region Change layout
        private void List_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            ADXOlExplorerLayout explorerLayout;
            try
            {
                explorerLayout = (ADXOlExplorerLayout)Enum.Parse(
                    typeof(AddinExpress.OL.ADXOlExplorerLayout), this.Item.ExplorerLayout.ToString()
                    );
            }
            catch
            {
                explorerLayout = ADXOlExplorerLayout.Unknown;
            }

            if (!SetExplorerLayout(userControl11.list.SelectedItem.ToString()))
            {
                userControl11.list.SelectionChanged -= List_SelectionChanged;
                userControl11.list.SelectedItem = this.Item.ExplorerLayout.ToString();
                userControl11.list.SelectionChanged += List_SelectionChanged;
            }
        }

        private enum OutlookPane
        {
            olFolderList = 2,
            olNavigationPane = 4,
            olOutlookBar = 1,
            olPreview = 3,
            olToDoPane = 5
        }
        private bool SetExplorerLayout(string layout)
        {
            ADXOlExplorerLayout newExplorerLayout;
            try
            {
                newExplorerLayout = (ADXOlExplorerLayout)Enum.Parse(
                    typeof(AddinExpress.OL.ADXOlExplorerLayout), layout
                    );
            }
            catch
            {
                newExplorerLayout = ADXOlExplorerLayout.Unknown;
            }
            if (newExplorerLayout == ADXOlExplorerLayout.Unknown)
                return false;

            Outlook.Explorer explorer = theAddinModule.OutlookApp.ActiveExplorer();
            if (explorer != null)
            {
                try
                {
                    string failureMessage = string.Empty;
                    if (!IsExplorerRegionAvailable(explorer, newExplorerLayout, out failureMessage))
                    {
                        if (failureMessage != string.Empty)
                            MessageBox.Show(failureMessage, "Can't change the Layout", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }

                    this.Item.ExplorerLayout = newExplorerLayout;
                    if (newExplorerLayout == AddinExpress.OL.ADXOlExplorerLayout.WebViewPane)
                    {
                        RefreshCurrentFolder(explorer);
                        this.Item.ApplyTo(explorer);
                    }
                    else
                    {
                        Outlook.MAPIFolder currentFolder = explorer.CurrentFolder;
                        if (currentFolder != null)
                        {
                            try
                            {
                                bool NeedClearFolderWebViewProperties =
                                    currentFolder.WebViewOn && UrlPointsToAdxfile(currentFolder.WebViewURL);
                                if (NeedClearFolderWebViewProperties)
                                {
                                    try
                                    {
                                        currentFolder.WebViewURL = string.Empty;
                                        currentFolder.WebViewOn = false;
                                    }
                                    catch
                                    {
                                    }
                                    RefreshCurrentFolder(explorer);
                                }
                                this.Item.ApplyTo(explorer);
                                return true;
                            }
                            finally
                            {
                                Marshal.ReleaseComObject(currentFolder);
                            }
                        }
                    }
                }
                finally
                {
                    Marshal.ReleaseComObject(explorer);
                }
            }
            return false;
        }

        private bool UrlPointsToAdxfile(string webViewURL)
        {
            bool result = CultureInfo.InvariantCulture.CompareInfo.IndexOf(webViewURL, ADXHTMLFileName, CompareOptions.IgnoreCase) >= 0;
            return result;
        }

        private bool IsExplorerRegionAvailable(Outlook.Explorer Explorer, AddinExpress.OL.ADXOlExplorerLayout ExplorerLayout, out string message)
        {
            bool result = true;
            message = string.Empty;

            switch (ExplorerLayout)
            {
                case AddinExpress.OL.ADXOlExplorerLayout.LeftSubpane:
                case AddinExpress.OL.ADXOlExplorerLayout.RightSubpane:
                case AddinExpress.OL.ADXOlExplorerLayout.TopSubpane:
                case AddinExpress.OL.ADXOlExplorerLayout.BottomSubpane:
                case AddinExpress.OL.ADXOlExplorerLayout.FolderView:
                    {
                        if (IsCalendarFolder(Explorer) & theAddinModule.HostMajorVersion >= 14)
                        {
                            result = false;
                            message = "The " + ExplorerLayout.ToString() + " layout is not supported in Calendar folders of Outlook 2010+. You can use the DockTop, DockBottom, DockRight and DockLeft regions as an alternative.";
                        }
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.WebViewPane:
                    {
                        result = false;
                        message = "The WebViewPane layout doesn't work by default. To find why this is so and how to deal with this, open the Resources form and see the corresponding hyperlink.\n\n" +
                            "Do you want to open that link?";
                        if (MessageBox.Show(message, "Navigate to web resource?", MessageBoxButtons.YesNo, MessageBoxIcon.Question ) == DialogResult.Yes)
                        {
                            System.Diagnostics.Process.Start("https://www.add-in-express.com/creating-addins-blog/2017/12/01/outlook-ignore-webviewurl-webviewon/");
                        }
                        message = string.Empty;
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.BottomNavigationPane:
                    {
                        switch (theAddinModule.HostMajorVersion)
                        {
                            case 9:
                            case 10:
                                {
                                    result = IsExplorerPaneVisible(Explorer, OutlookPane.olFolderList);
                                    if (!result)
                                        message = "Enable the Folder List; see menu View | Folder List.";
                                    break;
                                }
                            case 11:
                            case 12:
                                {
                                    result = IsExplorerPaneVisible(Explorer, OutlookPane.olNavigationPane);
                                    if (!result)
                                        message = "Enable the Navigation Pane; see menu View | Navigation Pane menu or Press Alt + F1.";
                                    break;
                                }
                            case 14:
                            case 15:
                            case 16:
                                {
                                    result = IsExplorerPaneVisible(Explorer, OutlookPane.olNavigationPane);
                                    if (!result)
                                        message = "Enable the Navigation Pane; see View | Folder Pane or Press Alt + F1.";
                                    break;
                                }
                            default:
                                {
                                    result = false;
                                    break;
                                }
                        }
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.BottomOutlookBar:
                    {
                        switch (theAddinModule.HostMajorVersion)
                        {
                            case 9:
                            case 10:
                                {
                                    result = IsExplorerPaneVisible(Explorer, OutlookPane.olOutlookBar);
                                    if (!result)
                                        message = "Enable the Outlook Bar; see menu View | Outlook Bar.";
                                    break;
                                }
                            case 11:
                            case 12:
                            case 14:
                            case 15:
                            case 16:
                                {
                                    result = false;
                                    message = "Outlook Bar is available in 2000 and XP versions only.";
                                    break;
                                }
                            default:
                                {
                                    result = false;
                                    break;
                                }
                        }
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.BottomTodoBar:
                    {
                        switch (theAddinModule.HostMajorVersion)
                        {
                            case 9:
                            case 10:
                            case 11:
                                {
                                    result = false;
                                    message = "The To-Do bar is available starting from version 2007.";
                                    break;
                                }
                            case 12:
                            case 14:
                                {
                                    result = IsExplorerPaneVisible(Explorer, OutlookPane.olToDoPane);
                                    if (!result)
                                        message = "Enable the To-Do Bar. See View | To-Do Bar or press Alt + F2.";
                                    break;
                                }
                            case 15:
                            case 16:
                                {
                                    result = false;
                                    message = "The BottomTodoBar layout is supported in Outlook 2007-2010 only.";
                                    break;
                                }
                            default:
                                {
                                    result = false;
                                    break;
                                }
                        }
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.BottomReadingPane:
                case AddinExpress.OL.ADXOlExplorerLayout.LeftReadingPane:
                case AddinExpress.OL.ADXOlExplorerLayout.RightReadingPane:
                case AddinExpress.OL.ADXOlExplorerLayout.TopReadingPane:
                    {
                        result = IsExplorerPaneVisible(Explorer, OutlookPane.olPreview);
                        if (!result)
                        {
                            switch (theAddinModule.HostMajorVersion)
                            {
                                case 9:
                                case 10:
                                    {
                                        message = "Enable the Preview Pane. See menu View | Preview Pane.";
                                        break;
                                    }
                                case 11:
                                case 12:
                                case 14:
                                case 15:
                                case 16:
                                    {
                                        message = "Enable the Reading Pane; see View | Reading Pane.";
                                        break;
                                    }
                            }
                        }
                        break;
                    }
                case AddinExpress.OL.ADXOlExplorerLayout.DockBottom:
                case AddinExpress.OL.ADXOlExplorerLayout.DockLeft:
                case AddinExpress.OL.ADXOlExplorerLayout.DockRight:
                case AddinExpress.OL.ADXOlExplorerLayout.DockTop:
                    {
                        result = true;
                        break;
                    }
                default:
                    {
                        result = false;
                        break;
                    }
            }
            return result;
        }
        private bool IsExplorerPaneVisible(object Explorer, OutlookPane OlPane)
        {
            return Convert.ToBoolean(Explorer.GetType().InvokeMember("IsPaneVisible", BindingFlags.InvokeMethod, null, Explorer, new object[] { (int)OlPane }));
        }
        private void RefreshCurrentFolder(Outlook.Explorer Explorer)
        {
            _refreshCurrentFolderExecuting = true;
            try
            {
                Outlook.MAPIFolder currentFolder = Explorer.CurrentFolder;
                Outlook.NameSpace nSpace = theAddinModule.OutlookApp.GetNamespace("MAPI");
                Outlook.MAPIFolder outboxFolder = nSpace.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderOutbox);
                try
                {
                    if (theAddinModule.HostMajorVersion <= 9)
                        Explorer.CurrentFolder = outboxFolder;
                    else
                    {
                        Explorer.GetType().InvokeMember("SelectFolder", BindingFlags.InvokeMethod, null, Explorer, new object[] { outboxFolder });

                        Application.DoEvents();

                        if (theAddinModule.HostMajorVersion <= 9)
                        {
                            Explorer.CurrentFolder = currentFolder;
                        }
                        else
                        {
                            Explorer.GetType().InvokeMember("SelectFolder", BindingFlags.InvokeMethod, null, Explorer, new object[] { currentFolder });
                        }
                    }
                }
                finally
                {
                    if (nSpace != null)
                        Marshal.ReleaseComObject(nSpace);
                    if (currentFolder != null)
                        Marshal.ReleaseComObject(currentFolder);
                    if (outboxFolder != null)
                        Marshal.ReleaseComObject(outboxFolder);
                }
            }
            finally
            {
                _refreshCurrentFolderExecuting = false;
            }
        }
        private bool IsCalendarFolder(Outlook.Explorer explorer)
        {
            Outlook.MAPIFolder currentFolder = null;
            try
            {
                currentFolder = explorer.CurrentFolder;
            }
            catch { }

            if (currentFolder == null) return false;

            bool result = 
                currentFolder.DefaultItemType == Outlook.OlItemType.olAppointmentItem;
            Marshal.ReleaseComObject(currentFolder);

            return result;
        }
        #endregion

        #region Logging the events of the form
        private void ADXOlFormExplorer_ADXAfterFormShow()
        {
            string currentLayout = "Explorer layout = '" + this.Item.ExplorerLayout.ToString() + "'";
            Logging.Log.Add(this.Text, "Form is shown in " + currentLayout);
        }

        private void ADXOlFormExplorer_ADXAfterFormHide(object sender, ADXAfterFormHideEventArgs e)
        {
            Logging.Log.Add(this.Text, "Form is hidden");
        }
        #endregion

        #region Dpi-related
        private void ADXOlFormExplorer_ADXDPIChanged(object sender, ADXDPIChangedEventArgs e)
        {
            this.SuspendLayout();
            FixDpi();
            this.ResumeLayout(true);

        }
        private void FixDpi()
        {
            if (this.elementHost1.Child != null
                && this.elementHost1.Child is UserControlPaneInfo)
            {
                (this.elementHost1.Child as UserControlPaneInfo).FixDpi(this.DPI);
            }
        }
        #endregion


        #region Representation
        // set the backcolor of the region's header
        protected override Type GetContainerControlType()
        {
            return typeof(Helpers.MyContainerControl);
        }
        #endregion
    }

}
