using AddinExpress.OL;
using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace adx_ol_cs_PaneLocation
{
    public partial class ADXOlFormInspector : AddinExpress.OL.ADXOlForm
    {
        private static int counter = 0;
        private const string ADXHTMLFileName = "ADXOlFormGeneral.html";
        private adx_ol_cs_PaneLocation.AddinModule theAddinModule;
        private int id = 0;

        public ADXOlFormInspector()
        {
            InitializeComponent();
            id = counter++;
            userControl11.list.SelectionChanged += List_SelectionChanged;
            this.Text = "ADXOlForm2. ID=" + id.ToString();
            Logging.Log.Add(this.Text, "Constructor");
        }

        #region form events: setting up the controls and closing the form
        private void ADXOlFormInspector_ADXBeforeFormShow()
        {
            Logging.Log.Add(this.Text, "ADXBeforeFormShow");

            theAddinModule = adx_ol_cs_PaneLocation.AddinModule.CurrentInstance;

            this.elementHost1.MinimumSize = new System.Drawing.Size((int)userControl11.MinWidth, (int)userControl11.MinHeight);

            #region populate the list and get the info string
            string[] items = new string[] {
                "LeftSubpane", "TopSubpane", "RightSubpane", "BottomSubpane",
                "InspectorRegion", "CompleteReplacement"
                };
            userControl11.list.SelectionChanged -= List_SelectionChanged;
            userControl11.PopulateListAndSelectItem(items, this.Item.InspectorLayout.ToString());
            userControl11.list.SelectionChanged += List_SelectionChanged;

            string formType = "Form type = 'ADXOlForm2'";
            string currentLayout = "Inspector layout = '" + this.Item.InspectorLayout.ToString() + "'";
            string currentInstance = "Instance Id = " + id.ToString();
            string currentTime = "Created at: " + System.DateTime.Now.ToLongTimeString();
            string itemInfo = "ADXOlFormsCollectionItem: " + this.Item.Tag;
            string dragDrop = "Item.IsDragDropAllowed: " + this.Item.IsDragDropAllowed.ToString();
            userControl11.Info = formType + "\n" + currentInstance + "\n" + currentTime + "\n" + itemInfo + "\n" + dragDrop;
            #endregion
            Logging.Log.Add(this.Text, "Form will be shown in " + currentLayout);
        }
        private void ADXOlFormInspector_ADXCloseButtonClick(object sender, ADXCloseButtonClickEventArgs e)
        {
            Logging.Log.Add(this.Text, "Close button is clicked");
            string message = "" +
                "You can close the current form instance. \n\n" +
                "Or\n\n" +
                "You can close all from instances created by an ADXOlFromsCollectionItem AND prevent the item from creating other form instances.\n\n" +
                "Click Yes to close this form instance. Close No to disable the corresponing ADXOlFromsCollectionItem.";
            DialogResult dialogResult = MessageBox.Show(message, "What to close?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (dialogResult == DialogResult.Cancel) return;
            if (dialogResult == DialogResult.Yes) e.CloseForm = true;
            if (dialogResult == DialogResult.No) e.DisableItem = true;
        }
        #endregion

        #region Change layout
        private void List_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            ADXOlInspectorLayout newInspectorLayout;
            try
            {
                newInspectorLayout = (ADXOlInspectorLayout)Enum.Parse(
                    typeof(AddinExpress.OL.ADXOlInspectorLayout), userControl11.list.SelectedItem.ToString()
                    );
            }
            catch
            {
                newInspectorLayout = ADXOlInspectorLayout.Unknown;
            }
            SetInspectorLayout(newInspectorLayout);
        }

        private void SetInspectorLayout(ADXOlInspectorLayout newInspectorLayout)
        {
            if (newInspectorLayout == ADXOlInspectorLayout.Unknown) 
                return;

            bool isModernOutlook = theAddinModule.HostMajorVersion >= 12;

            this.Item.InspectorLayout = newInspectorLayout;

            bool canDoThis = isModernOutlook;
            if (canDoThis == false)
                // Outlook 2000-2003 will hang when OutlookApp.ActiveInspector() is called if Word is the default email editor.
                canDoThis = !IsAnInstanceVisible(this.Item);

            if (canDoThis)
            {
                Outlook.Inspector activeInspector = theAddinModule.OutlookApp.ActiveInspector();
                if (activeInspector != null)
                {
                    try
                    {
                        this.Item.ApplyTo(activeInspector);
                    }
                    finally
                    {
                        Marshal.ReleaseComObject(activeInspector);
                    }
                }
            }
        }
        private bool IsAnInstanceVisible(AddinExpress.OL.ADXOlFormsCollectionItem Item)
        {
            for (int i = 0; i <= Item.FormInstanceCount - 1; i++)
            {
                if (Item.FormInstances(i).Visible)
                    return true;
            }
            return false;
        }
        #endregion

        #region Logging the events of the form
        private void ADXOlFormInspector_ADXAfterFormShow()
        {
            Logging.Log.Add(this.Text, "ADXAfterFormShow");
        }

        private void ADXOlFormInspector_ADXAfterFormHide(object sender, ADXAfterFormHideEventArgs e)
        {
            Logging.Log.Add(this.Text, "ADXAfterFormHide");
        }
        #endregion

        #region Dpi-related
        private void ADXOlFormInspector_ADXDPIChanged(object sender, ADXDPIChangedEventArgs e)
        {
            this.SuspendLayout();
            FixDpi();
            this.ResumeLayout(true);
        }

        private void FixDpi()
        {
            if (this.elementHost1.Child != null
                && this.elementHost1.Child is UserControlPaneInfo)
            {
                (this.elementHost1.Child as UserControlPaneInfo).FixDpi(this.DPI);
            }
        }
        #endregion

        #region UI
        // set the backcolor of the region's header
        protected override Type GetContainerControlType()
        {
            return typeof(Helpers.MyContainerControl);
        }
        #endregion
    }
}
