﻿using System;
using System.Windows.Forms;

namespace DpiScaling
{
    public partial class MyDpiAwareForm : Form
    {
        public MyDpiAwareForm()
        {
            InitializeComponent();
            ScaleMe = false;
        }
        public MyDpiAwareForm(bool scaleMe)
        {
            InitializeComponent();
            ScaleMe = scaleMe;
        }
        internal virtual void RefreshInfo() { }
        internal virtual void FixDpi() { }

        public bool ScaleMe { get; private set; } = false;

        private int dpi = Win32Api.GetDpiForSystem();
        public int DPI
        {
            get
            {
                if (dpi == 0)
                    dpi = GetDPI();
                Logging.Log.Add("DPI.get", "dpi=" + dpi.ToString());
                return dpi;
            }
            internal set
            {
                if (dpi != value)
                {
                    Logging.Log.Add("DPI.set", "dpi=" + dpi.ToString());
                    dpiFactor = (float)value / (float)dpi;
                    dpi = value;

                    OnDpiChangedEvent();
                }
            }
        }
        private float dpiFactor = 1.0f;
        public float DPIFactor
        {
            get { return dpiFactor; }
        }

        private int GetDPI()
        {
            int result = int.MinValue;
            if (this.Handle != IntPtr.Zero)
            {
                Logging.Log.Add("GetDPI", "dpi=" + dpi.ToString() + " " + "this.Handle != IntPtr.Zero");
                result = (int)Win32Api.GetDpiForWindow(this.Handle);
            }
            else
            {
                Logging.Log.Add("GetDPI", "dpi=" + dpi.ToString() + " " + "this.Handle == IntPtr.Zero");
                result = Win32Api.GetDpiForSystem();
            }
            return result;
        }

        private void OnDpiChangedEvent()
        {
            Logging.Log.Add("OnDpiChangedEvent", "dpi=" + dpi.ToString());

            this.SuspendLayout();
            FixDpi();
            this.ResumeLayout(true);
            RefreshInfo();
        }
    }
}
